/*******************************************************************************************************
<SCRIPTFILE>
Created: 2013-00-00
Last Updated: 2015-10-08
Version: 0.10

Author: Klaas Nienhuis, mail@klaasnienhuis.nl, www.klaasnienhuis.nl
Version: 3ds max 2012

Description:
	The installer that comes with the mzp-builder
Usage:
	RUN IT

Features:

Wishlist:
Changelog:
</SCRIPTFILE>
*******************************************************************************************************/

try(destroyDialog mzpInstaller.gui.roll_installer)catch()


(
	clearListener()
	
	--try to get the mzpBuilderInterface
	--we need it to parse the manifest
	local thePath = (getFilenamePath (getSourceFileName())) + @"payload\scriptfiles\builder\mzpBuilder_interface.ms*"
	local arrFile = getfiles thePath
	if arrFile.count > 0 do filein arrFile[1]
		
	--initialize the main struct
	struct str_mzpInstaller
	(
		basePath = getFilenamePath (getSourceFileName()), --the folder this script is located in
			
		--structs
		gui,
		uninstall,
		dataContainerStruct,
		data, 
			
		--variables
		guiColors,
			
		--filepaths and folderpaths
		payloadPath = "payload",
		manifestPath,
		configPath
	)
	
	global mzpInstaller  = str_mzpInstaller()
)

(
	struct str_dataContainer
	(
		scriptName = "undefined name",
		version = "undefined version",
		category = "undefined category",
		description = "",
		installmessage = "",
		isDevelopment = false,
		deployprojectroot = "",
		buildfilename = "",
		doEncrypt = true,
		structname = "",
		installertitle = "",
		installertutorialurl = "http://www.klaasnienhuis.nl/2015/02/install-maxscript-with-mzp-file/",
		
		function fn_harvestData manifestPath configPath =
		(
			/*<FUNCTION>
			Description
				harvests data from mzpmanifest and config and stores the data in this struct
			Arguments
				<string> manifestPath: the path to the manifestPath-file
				<string> configPath: the path to the configPath-file
			Return
			<FUNCTION>*/
			
			--extract the data from the xml files
			local theContainer = mzpBuilderInterface.dataContainer()
			mzpBuilderInterface.interface.fn_readMzpManifest manifestPath theContainer	
			mzpBuilderInterface.interface.fn_readConfig configPath theContainer	
			
			--build parameters
			deployprojectroot = theContainer.buildparameters.item["deployprojectroot"]
			buildfilename = theContainer.buildparameters.item["buildfilename"]
			category = theContainer.buildparameters.item["category"]
			structname = theContainer.buildparameters.item["structname"]
			doEncrypt = theContainer.buildparameters.item["encrypt"]
			
			--optional parameters
			if theContainer.aboutscript.item["name"] != undefined do scriptName = theContainer.aboutscript.item["name"]
			if theContainer.aboutscript.item["installMessage"] != undefined do installMessage = theContainer.aboutscript.item["installMessage"]
				
			if theContainer.scriptconfig.item["version"] != undefined do version = theContainer.scriptconfig.item["version"]
			if theContainer.scriptconfig.item["development"] != undefined do isDevelopment = theContainer.scriptconfig.item["development"]
			if theContainer.scriptconfig.item["prettyname"] != undefined do scriptName = theContainer.scriptconfig.item["prettyname"]
			if theContainer.scriptconfig.item["installertitle"] != undefined do installertitle = theContainer.scriptconfig.item["installertitle"]
			if theContainer.scriptconfig.item["installertutorialurl"] != undefined do installertutorialurl = theContainer.scriptconfig.item["installertutorialurl"]
		)
	)
	
	struct str_uninstall
	(
		parentStruct,
		dirIO = dotNetClass "system.IO.directory",
		
		function fn_cleanupTemp =
		(
			/*<FUNCTION>
			Description:
				Cleans the temp folder where the installer and payload have been extracted to.
			Arguments:
			Return:
			</FUNCTION>*/
			local outputMessage = stringstream ""
			
			if parentStruct.data.buildFileName != "" do
			(
				local thePath = symbolicPaths.expandFileName ((dotnetClass "System.IO.Path").Combine #("$temp", parentStruct.data.buildFileName))
				format "Temp path %\n" thePath to:outputMessage
				if this.dirIO.exists thePath do 
				(
					try(this.dirIO.delete thePath true)catch
					(
						format "Couldn't delete all temp files, maybe they're still open." to:outputMessage
						parentStruct.gui.fn_addMessage theMessage:outputMessage
					)
				)
			)
		),
		
		function fn_cleanupTarget skipQuery:false =
		(
			/*<FUNCTION>
			Description:
				Cleans the target-folder where a previous script might have been installed before
			Arguments:
				<boolean> skipQuery: set to true if you want to skip a query
			Return:
			</FUNCTION>*/
			
			--try to close the old script gui if it's still open
			--this should avoid files still being open while updating
			local theOldScript = execute parentStruct.data.structname
			if theOldScript != undefined do
			(
				try(theOldScript.guiForm.form_main.close())catch() --most dotnet based script use this gui structure
			)
			
			closeLog()
			
			--by default the tool cleans up the installation folder. If there are any files created at runtime such as cookies or api-keys
			--stored outside the installation folder the installed script may keep track of that in the ini-file in a section called "runtimefiles"
			--delete those too
			local installedConfigXml = (dotnetClass "System.IO.Path").Combine #(parentStruct.data.deployprojectroot, "config","config.xml")
			if doesFileExist installedConfigXml do
			(
				local runtimeHash = mzpBuilderInterface.interface.fn_getRuntimeFiles (symbolicPaths.expandFileName installedConfigXml)
				if runtimeHash.count > 0 do
				(
					local enum = runtimeHash.GetEnumerator()
					while enum.MoveNext() do 
					(
						format "Deleting runtimefile %\r\n" enum.Value
						if doesFileExist enum.Value do deleteFile enum.Value
					)		
				)			
			)
			
			--Delete teh enitre installation folder
			local thePath = symbolicPaths.expandFileName parentStruct.data.deployprojectroot
			local proceed = true
			if NOT this.dirIO.exists thePath do
			(
				proceed = false
				if NOT skipQuery do parentStruct.gui.fn_addMessage theMessage:("Can't remove existing files. The path doesn't exist.\n\n" + thePath)
			)
			if proceed AND NOT skipQuery do
			(
				proceed = queryBox ("Proceed to uninstall " + thePath + " agreed?") title:"Uninstall folder"
			)
			if proceed then 
			(
				try
				(
					this.dirIO.delete thePath true
					if NOT skipQuery do parentStruct.gui.fn_addMessage theMessage:"Script uninstalled. You can re-install at anytime with the same installer file."
				)catch
				(
					parentStruct.gui.fn_addMessage theMessage:"Couldn't delete all files, maybe they're still open.\n"
				)
			)
			proceed
		)
	)

	struct str_gui
	(
		parentStruct,
		
		roll_installer,
		
		function fn_getRolloutBackForeColor &colorStruct type:#dotnet =
		(
			/*<FUNCTION>
			Description
				gets colors from the current color scheme. Returns either max or dotnet colors
				kudos to Johan Boekhoven
			Arguments
				<struct by reference> colorStruct: will hold the colors
				<name> #type: the type of output color #max | #dotnet
			Return
			<FUNCTION>*/
			
			struct str_colors
			(
				backColor,
				textColor,
				threedLightColor,
				window,
				shadow,
				appWorkspace,
				threedDarkShadow
			)
			colorStruct = str_colors()
			colorStruct.backColor = colorMan.getColor #background
			colorStruct.textColor = colorMan.getColor #text
			colorStruct.threedLightColor = colorMan.getColor #3dLight
			colorStruct.window = colorMan.getColor #window
			colorStruct.threedDarkShadow = colorMan.getColor #3dDarkShadow
			colorStruct.appWorkspace = colorMan.getColor #appWorkspace
			colorStruct.shadow = colorMan.getColor #shadow
			
			
			local arrPropName = getPropNames colorStruct
			
			--colorman returns colors are point3 values. Let's convert them to dotnet colors
			for thePropName in arrPropname do
			(
				local p3 = getProperty colorStruct thePropName
				setProperty colorStruct thePropName ((dotNetClass "System.Drawing.Color").FromArgb 255 (p3.x * 255.) (p3.y * 255.) (p3.z * 255.))
			)
			
			--optionally convert the colors into max colors
			if type == #max do 
			(
				for thePropName in arrPropname do
				(
					local dotnetColor = getProperty colorStruct thePropName
					setProperty colorStruct thePropName (color dotnetColor.r dotnetColor.g dotnetColor.b)
				)
			)
		),
		
		function fn_addMessage theMessage:"" =
		(
			/*<FUNCTION>
			Description:
				Displays a message in the log.
			Arguments:
				<string> theMessage: the message you want to display
			Return:
			</FUNCTION>*/

			messageBox (theMessage as string) title:"Script installer message"
			format "%" (theMessage as string)
		),
		
		function fn_buttonFactory theButton theText:  =
		(
			/*<FUNCTION>
			Description
				sets a specific .net button style
			Arguments
				<button control> theButton: the button to style
				<string> theText: the text in the button
			Return
			<FUNCTION>*/
			
			if theText != unsupplied do theButton.Text = theText

			theButton.forecolor = parentStruct.guiColors.threedLightColor
			theButton.backcolor = parentStruct.guiColors.appWorkspace
			theButton.flatStyle = theButton.flatStyle.flat
			theButton.FlatAppearance.BorderColor = parentStruct.guiColors.threedLightColor
		),			
		
		function fn_defineRollouts =
		(
			/*<FUNCTION>
			Description:
				Defines the rollouts.
			Arguments:
			Return:
			</FUNCTION>*/
			
			rollout roll_installer "Script installer - Klaas Nienhuis - www.klaasnienhuis.nl" width:350 height:230
			(
				local parentStruct = mzpInstaller
				
				dotnetcontrol lblBanner "label" width:348 align:#left height:70 offset:[-12,-4]
				dotnetcontrol btnInstall "Button" width:160 height:40 align:#left offset:[-6,0]
				dotnetcontrol btnUninstall "Button" width:160 height:22 align:#left offset:[-6,0]
				dotnetcontrol btnShowReadme "Button" width:160 height:22 align:#left offset:[-6,0]
				dotnetcontrol btnTutorial "Button" width:160 height:22 align:#left offset:[-6,0] --visible:false
				dotnetcontrol btnClose "Button" width:160 height:22 align:#left offset:[-6,0]
				dotnetcontrol lblSummary "label" width:168 height:148 pos:[174,76]
				
				on roll_installer open do
				(
					parentStruct.gui.fn_getRolloutBackForeColor &parentStruct.guiColors type:#dotnet	

					--load data from the config and mzpmanifest files
					parentStruct.manifestPath = (dotnetClass "System.IO.Path").Combine #(parentStruct.basePath, "mzpmanifest.xml")
					parentStruct.configPath = (dotnetClass "System.IO.Path").Combine #(parentStruct.basePath, "config.xml")
					parentStruct.data = parentStruct.dataContainerStruct()
					parentStruct.data.fn_harvestData parentStruct.manifestPath parentStruct.configPath

					if parentStruct.data.installertitle != "" do roll_installer.title = parentStruct.data.installertitle
						
					--style the labels
					lblBanner.backColor = parentStruct.guiColors.backColor
					lblBanner.forecolor = parentStruct.guiColors.textcolor
					lblBanner.text = parentStruct.data.scriptName
					lblBanner.Font = dotnetObject "system.drawing.Font" "Arial" 20
					lblBanner.TextAlign = (dotnetclass "System.Drawing.ContentAlignment").MiddleCenter
					
					local strSummary = stringStream ""
					format "Name: %\r\nVersion: %\r\nCategory: %\r\n" parentStruct.data.scriptName parentStruct.data.version parentStruct.data.category to:strSummary
					if parentStruct.data.isDevelopment do format "Development build\r\n" to:strSummary
					format "\r\n%" parentStruct.data.installMessage to:strSummary
					
					lblSummary.text = (strSummary as string)
					lblSummary.borderStyle = (dotnetClass "system.Windows.Forms.BorderStyle").FixedSingle
					lblSummary.backColor = parentStruct.guiColors.backColor
					lblSummary.forecolor = parentStruct.guiColors.textcolor

					--style the buttons
					parentStruct.gui.fn_buttonFactory btnInstall theText:"Install script"
					btnInstall.forecolor = (dotNetClass "System.Drawing.Color").FromArgb 0 0 0
					btnInstall.backcolor = ((dotNetClass "System.Drawing.Color").FromArgb 150 200 80)
					btnInstall.Font = dotnetObject "system.drawing.Font" "Arial" 15 
					
					parentStruct.gui.fn_buttonFactory btnUninstall theText:"Uninstall"
					parentStruct.gui.fn_buttonFactory btnShowReadme theText:"Show files"
					parentStruct.gui.fn_buttonFactory btnTutorial theText:"Customize UI mini tutorial"
					parentStruct.gui.fn_buttonFactory btnClose theText:"Close"
					
					--get data from the previously installed script 
					getDirectories (parentStruct.data.deployprojectroot + "*")
					if (getDirectories (parentStruct.data.deployprojectroot + "*")).count == 0 do
					(
						btnUninstall.enabled = false
					)
				)
				
				on roll_installer close do
				(
					mzpInstaller.uninstall.fn_cleanupTemp()
				)
				
				on btnInstall mouseclick do
				(
					local proceed = parentStruct.uninstall.fn_cleanupTarget skipQuery:true

					local hasIssues = false
					local copyIssues = 0
					local deleteIssues = 0
					local outputMessage = stringstream ""
					local deleteOK
					local copyOK
					
					/*********************************************************************************
					COPY FILES FROM MANIFEST TO DESTINATION
						also copy the configfile and macro
						run the macro
					*********************************************************************************/
					local payloadPath = (dotnetClass "System.IO.Path").Combine #(parentStruct.basePath, parentStruct.payloadPath)
					
					if doesFileExist parentStruct.manifestPath then
					(
						--copy the assets as specified in the xml-file
						local xmlDoc = dotNetObject "system.xml.xmlDocument"
						xmlDoc.load parentStruct.manifestPath

						format "Using xml, file is present: %\n" (doesfileExist parentStruct.manifestPath)

						/*********************************************************************************
						BUILD PAYLOAD ASSETS
						*********************************************************************************/
						local arrSource = xmlDoc.selectNodes "//paths/sources/source"
						format "found % sources\n" arrSource.count
						local strXpath = "" as stringstream
						for i = 0 to arrSource.count-1 do
						(
							--collect data from this source
							local relativeSourcePath = (arrSource.itemOf[i].selectSingleNode "relbuildpath").innerText
							local canEncrypt = (arrSource.itemOf[i].selectSingleNode "mxsencrypt").innerText as booleanclass
							local stagingSource = (dotnetClass "System.IO.Path").Combine #(payloadPath, relativeSourcePath)							
							local filename = (arrSource.itemOf[i].selectSingleNode "filename").innerText
							local arrTargetIndex = execute (arrSource.itemOf[i].selectSingleNode "targetidarray").innerText
							
							--loop over every target for this source and install the source in each target
							for n = 1 to arrTargetIndex.count do
							(
								strXpath = "" as stringstream
								format "//paths/targets/target[@id='%']/path" arrTargetIndex[n] to:strXpath
								local relativeDestinationPath = (xmlDoc.selectSingleNode strXpath).innerText
								strXpath = "" as stringstream
								format "//paths/targets/target[@id='%']/type" arrTargetIndex[n] to:strXpath
								local targetType = (xmlDoc.selectSingleNode strXpath).innerText
								

								local theDestination = case targetType of
								(
									"absolute": 
									(
										(dotnetClass "System.IO.Path").Combine #(relativeDestinationPath, filename)
									)
									"mxs": 
									(
										(dotnetClass "System.IO.Path").Combine #((symbolicPaths.expandFileName relativeDestinationPath), filename)
									)
									"windows": 
									(
										--TODO:make sure to remove the pluginfile first. the NW _installer file has nice methods for this.
										local arrPath = filterString relativeDestinationPath "%" --separate the windows-part from the rest of the path
										local windowsSpecialFolder
										case arrPath[1] of
										(
											"ApplicationData": windowsSpecialFolder = (dotnetClass "System.Environment+SpecialFolder").ApplicationData
										)
										local theEnvironmentPath = (dotnetClass "System.Environment").GetFolderPath windowsSpecialFolder
										(dotnetClass "System.IO.Path").Combine #(theEnvironmentPath, arrPath[2], filename)
									)
								)
								
								makeDir (getFilenamePath theDestination) all:true
								if parentStruct.data.doEncrypt AND canEncrypt do
								(
									stagingSource = (dotnetClass "system.io.Path").ChangeExtension stagingSource ".mse"
									theDestination = (dotnetClass "system.io.Path").ChangeExtension theDestination ".mse"
								)
								
								deleteOK = true
								if doesFileExist theDestination do deleteOK = deleteFile theDestination
								copyOK = copyFile stagingSource theDestination
								
								if NOT deleteOK do 
								(
									hasIssues = true
									deleteIssues += 1
									format "Delete error\n\tdestination %\n" theDestination to:outputMessage
								)
								if NOT copyOK do 
								(
									hasIssues = true
									copyIssues += 1
									format "Copy error\n\tsource: %\n\tdestination %\n" stagingSource theDestination to:outputMessage
								)
							)
						)
						
					)else
					(
						--copy the payload into place
						for i = 1 to parentStruct.data.arrSource.count do
						(
							local theSource = (dotnetClass "System.IO.Path").Combine #(payloadPath, (getINISetting parentStruct.iniPath "Source" parentStruct.data.arrSource[i]))
							local theDestination = (dotnetClass "System.IO.Path").Combine #((symbolicPaths.expandFileName (getINISetting parentStruct.iniPath "Destination" parentStruct.data.arrDestination[i])), (filenameFromPath theSource))
							makeDir (getFilenamePath theDestination)
							if parentStruct.data.doEncrypt AND findItem parentStruct.data.encryptIndices (parentStruct.data.arrSource[i] as integer) > 0 do
							(
								theSource = (dotnetClass "system.io.Path").ChangeExtension theSource ".mse"
								theDestination = (dotnetClass "system.io.Path").ChangeExtension theDestination ".mse"
							)
							local deleteOK = deleteFile theDestination
							local copyOK = copyFile theSource theDestination
							
							if NOT copyOK do 
							(
								hasIssues = true
								format "Copy error%\n" theSource to:outputMessage
							)
						)
					)
					
					local deployprojectroot = symbolicPaths.expandFileName parentStruct.data.deployprojectroot
					
					--copy the config file
					local targetPath = (dotnetClass "System.IO.Path").Combine #( deployprojectroot, "config", "config.xml")
					makeDir (getFilenamePath targetPath)
					copyFile parentStruct.configPath targetPath
					
					--copy the manifest builder file
					local targetPath = (dotnetClass "System.IO.Path").Combine #( deployprojectroot, "scriptfiles", "builder", "mzpBuilder_interface.ms")
					makeDir (getFilenamePath targetPath)
					copyFile ((dotnetClass "System.IO.Path").Combine #( payloadPath, "builder", "mzpBuilder_interface.ms")) targetPath
					
					--copy and run the macro
					local targetPath = (dotnetClass "System.IO.Path").Combine #(deployprojectroot, "installer", "makeMacro.ms")
					makeDir (getFilenamePath targetPath)
					copyFile ((dotnetClass "System.IO.Path").Combine #(payloadPath, "installer", "makeMacro.ms")) targetPath
					fileIn targetPath
					
					if hasIssues then
					(
						format "Script installed with issues.\n\tCopy issues: %\n\tDelete issues: %\n" copyIssues deleteIssues to:outputMessage
					)else
					(
						format "% % has now been installed!\n\n" parentStruct.data.scriptName parentStruct.data.version to:outputMessage
						format "To set it up go to:\n" to:outputMessage
						format "Menu -> Customize -> Customize User Interface -> Toolbars\nCategory: \"%\" and drag \"%\" to a toolbar.\n\n" parentStruct.data.category parentStruct.data.scriptName to:outputMessage
						format "Click the toolbar icon to run the script.\n\n" to:outputMessage
						format "Or check out the mini tutorial to see how to add the script to your toolbar." to:outputMessage
					)
					
					parentStruct.gui.fn_addMessage theMessage:outputMessage
				)
				
				on btnUninstall mouseclick do 
				(
					mzpInstaller.uninstall.fn_cleanupTarget skipQuery:false
				)
				
				
				on btnShowReadme mouseclick do
				(
					ShellLaunch "notepad" ((dotnetClass "System.IO.Path").Combine #((getDir #temp), parentStruct.data.buildfilename, "readme.txt"))
				)
					
				on btnTutorial mouseclick do 
				(
					ShellLaunch parentStruct.data.installertutorialurl ""
				)
					
				on btnClose mouseclick do
				(
					try(DestroyDialog parentStruct.gui.roll_installer)catch()
				)
			)
		),
		
		define = fn_defineRollouts()
	)
	
	mzpInstaller.uninstall = str_uninstall parentStruct:mzpInstaller
	mzpInstaller.gui = str_gui parentStruct:mzpInstaller
	mzpInstaller.dataContainerStruct = str_dataContainer
)


createDialog mzpInstaller.gui.roll_installer style:#(#style_toolwindow, #style_sysmenu)